@@ -45,7 +45,7 @@ def parse_dpkg_info(content):
                         current['installed'] = False
                 elif interesting in ('Depends:', 'Pre-Depends:'):
                     value = [x.groupdict() for x in DEPENDENCY_MATCHER.finditer(value)]
-                    if interesting is 'Depends:':
+                    if interesting == 'Depends:':
                         key = 'depends'
                     else:
                         key = 'predepends'
